package be.iminds.ilabt.jfed.highlevel.util;

import be.iminds.ilabt.jfed.experiment.Experiment;
import be.iminds.ilabt.jfed.rspec.model.ModelRspecType;
import be.iminds.ilabt.jfed.rspec.model.RspecInterface;
import be.iminds.ilabt.jfed.rspec.model.RspecNode;
import be.iminds.ilabt.jfed.rspec.model.javafx_impl.FXModelRspec;
import be.iminds.ilabt.jfed.util.GeniUrn;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.stream.Collectors;

/**
 * ESpecMetaFileCreator
 */
public class ESpecMetaFileCreator {
    private final Experiment experiment;

    private final static ObjectMapper MAPPER = new ObjectMapper();

    public ESpecMetaFileCreator(Experiment experiment) {
        this.experiment = experiment;
    }

    public byte[] getManifestXml() {
        if (experiment.getSlice().getManifestRspec() == null)
            throw new IllegalStateException("There is no manifest while uploading manifest.xml to node");
        String manifestString = experiment.getSlice().getManifestRspec().getRspecXmlString();
        if (manifestString == null)
            throw new IllegalStateException("There is no manifest string while uploading manifest.xml to node");
        return manifestString.getBytes(StandardCharsets.UTF_8);
    }

    public byte[] getExperimentInfoJson() {
        if (experiment.getSlice().getManifestRspec() == null)
            throw new IllegalStateException("There is no manifest while uploading manifest.xml to node");
        FXModelRspec modelRspec = experiment.getSlice().getManifestRspec().getModelRspec(ModelRspecType.FX);
        if (modelRspec == null)
            throw new IllegalStateException("There is no parsable manifest while generating meta experiment-info.json");

        Map<String, Map<String, Map<String, Object>>> res = new HashMap<>();
        Map<String, Map<String, Object>> nodes = new HashMap<>();
        res.put("nodes", nodes);

        for (RspecNode node : modelRspec.getNodes()) {
            if (node.getClientId() == null)
                continue;

            Map<String, Object> nodeInfo = new HashMap<>();
            nodeInfo.put("client_id", node.getClientId());
            if (node.getAnsibleGroups() == null || node.getAnsibleGroups().isEmpty()) {
                nodeInfo.put("ansible_groups", Collections.emptyList());
            } else {
                nodeInfo.put("ansible_groups", new ArrayList<>(node.getAnsibleGroups()));
            }
            if (node.getInterfaces() == null || node.getInterfaces().isEmpty()) {
                nodeInfo.put("internal_ips", Collections.emptyList());
            } else {
                List<String> ips = node.getInterfaces().stream()
                        .map(RspecInterface::getIpAddresses)
                        .filter(Objects::nonNull)
                        .flatMap(Collection::stream)
                        .map(RspecInterface.IpAddress::getAddress)
                        .filter(Objects::nonNull)
                        .collect(Collectors.toList());
                nodeInfo.put("internal_ips", ips);
            }
            nodes.put(node.getClientId(), nodeInfo);
        }

        try {
            return MAPPER.writeValueAsBytes(res);
        } catch (JsonProcessingException e) {
            throw new RuntimeException("Problem generating JSON while generating meta experiment-info.json", e);
        }
    }
}
